@@ -61,14 +61,52 @@
     >>> cs.pat_enable = 1 #Turn on special pattern trigger
     >>> cs.pulse = 1
     >>> cs.armed = False #Can use true/false or 1/0
+    
+    Scripting
+    ---------
+    
+    Scripting ChipSHOUTER is recommended to deal with the expected error conditions. In
+    particular, the ChipSHOUTER may reset during use due to the EMFI events. This can
+    be detected as shown here::
+    
+        from chipshouter import ChipSHOUTER
+        from chipshouter.com_tools import Reset_Exception
+        cs = ChipSHOUTER('COM4')
+
+        def setup_device:
+            #Set any defaults you want
+            cs.pulse.repeat = 1
+
+            #Arm - add some delay for next stuff
+            cs.armed = 1
+            time.sleep(0.5)
+            
+            #Turn off sound if you want
+            #cs.mute = True
+
+        while True:
+            try:
+                old = cs.voltage.set
+                cs.voltage = random.randint(200, 500)
+                cs.pulse = 1
+            except Reset_Exception:
+                print("Device rebooted!")
+                time.sleep(5) #Need to wait after reboot!
+                setup_device()
 
 '''
 from collections import OrderedDict
+import re
+import logging
 from .com_tools import Bin_API
 from .com_tools import Reset_Exception
 from .com_tools import Max_Retry_Exception
+from .console.serial_interface import Serial_interface
+from .console.console import Console
 import time
 
+firmware_version = '2.0.2'
+
 api_version = '0.0.0'
 
 class DisableNewAttr(object):
@@ -292,6 +330,7 @@ class ChipSHOUTER(DisableNewAttr):
         self._voltage = VoltageSettings(self.com_api)
 
         self.disable_newattr()
+        self._check_fw_version()
 
     def status(self):
         """ This will indicate the status of the connection to the ChipSHOUTER.
@@ -400,7 +439,7 @@ class ChipSHOUTER(DisableNewAttr):
 
     @property
     def pulse(self):
-        """ Sets parameters for the pulse generator and causes the pulse
+        r""" Sets parameters for the pulse generator and causes the pulse
         to be triggered.
         
         :param pulse: (bool): When set to True causes a pulse to occur.
@@ -539,12 +578,20 @@ class ChipSHOUTER(DisableNewAttr):
     @property
     def trigger_safe(self):
         """
-        This will let you know if it is safe to trigger. 
-        If this is read and the system is safe it will allow triggering for 5 seconds
-        without checking the temperature. External triggers without calling this first
-        are not guaranteed to be successful. The unit could have a fault and abort triggering. 
+        Informs the ChipShouter that it is safe to read sensors.
 
-        :Returns (bool): - True to indicat that it is safe to trigger. 
+        Typically, the ChipShouter will read a temperature sensors
+        every few hundred ms. This can fail during a discharge 
+        event. Calling trigger_safe while the ChipShouter is armed
+        will do an immediate sensor read, then disable temperature
+        sensor reads. This must be called periodically, determined
+        by absent_temp, to read the temperature sensor. If absent_temp
+        elapses without a temperature read, the ChipShouter will fault.
+
+        This manual temperature read mode will be disabled upon
+        disarming the scope.
+
+        :Returns (bool): - The sensors have been read and trigger_safe is enabled
                          - False if the system is not armed or in fault.
                          
         """
@@ -654,6 +701,16 @@ class ChipSHOUTER(DisableNewAttr):
         """
         return self.com_api.get_id()
 
+    def _check_fw_version(self):
+        cs_id = self.id
+        cs_ver = re.search(r"[0-9]\.[0-9]\.[0-9]", cs_id).group(0)
+        if cs_ver < firmware_version:
+            logging.warning("-------------------------------------------------------")
+            logging.warning("New firmware update available (cur: {}) (new: {})".format(cs_ver, firmware_version))
+            logging.warning("Please contact support@newae.com for updated firmware")
+            logging.warning("-------------------------------------------------------")
+
+
     @property
     def arm_timeout(self):
         """ Arm timeout 
@@ -770,7 +827,8 @@ class ChipSHOUTER(DisableNewAttr):
 
     @property
     def absent_temp(self):
-        """Configure maximum time the temperature sensors can be skipped for.
+        """Configure maximum time the temperature sensors can be skipped for
+        during trigger_safe mode.
         The temperature sensors cannot be read during pulse events, and
         the ChipSHOUTER keeps a timer of how old the last temperature reading
         is.
@@ -800,15 +858,14 @@ class ChipSHOUTER(DisableNewAttr):
 
     @property
     def pat_wave(self):
-        """The wave used for the pattern trigger when pat_enable is 'True'.
+        r"""The wave used for the pattern trigger when pat_enable is 'True'.
 
         When reading the This will return the string up to 96 bits.
 
         If the length is more than 96 bits the string will append '... ( 4904 more )'
         >>> cs.pat_wave
-        '111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111 ... ( 4904 more )'
-         >>>
-
+        '11111111111111111111111111111111111111...111111111111111 ... ( 4904 more )'
+        >>>
         **NOTE**: To get pattern waves that are longer than 96 use the get_pat_wave function.
 
         example:
@@ -817,9 +874,9 @@ class ChipSHOUTER(DisableNewAttr):
          '11001'
          >>> cs.pat_wave = '1'*97
          >>> cs.pat_wave
-         '111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111 ... ( 1 more )'
+         '111111111111111111111111111111111111111...11111111111111111 ... ( 1 more )'
          >>> cs.get_pat_wave()
-         '1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111'
+         '111111111111111111111111111111111111111...11111111111111111'
          >>>
 
         **NOTE**: You **MUST** end the pattern with an inactive value,
@@ -886,6 +943,16 @@ class ChipSHOUTER(DisableNewAttr):
             self.com_api.cmd_reset()
         return
 
+    def eeprom_write(self):
+        """Write settings to EEPROM
+
+        Originally, everytime config was changed, it was
+        written to EEPROM, which could destroy EEPROM after a while.
+
+        Now only saves to EEPROM if this is called
+        """
+        self.com_api.cmd_eeprom_write()
+
     @property
     def boot_enable(self):
         return 0
@@ -913,6 +980,41 @@ class ChipSHOUTER(DisableNewAttr):
     def board_id(self):
         return self.com_api.get_board_id()
 
+    def update_firmware(self, file, comport=None):
+        """Updates the firmware of the connected ChipSHOUTER
+
+        Supports both .fup and .bin files. If a .fup file is used,
+        will verify that the firmware and ChipSHOUTER board IDs match.
+
+        Args:
+            file (str): Path to the firmware file
+            comport (str): Optional if using function stand-alone, specify comport
+        """
+        if comport is None:
+            apid = True
+            comport = self.com_api.comport
+            self.com_api.ctl_disconnect()
+        else:
+            apid = False
+        serial = Serial_interface(use_threads = False)
+        if serial.s_open(comport, timeout=0.01):
+            print("Connection successful")
+        else:
+            raise OSError("Unable to open serial port")
+        my_console = Console(serial, threads=False, sendfile=file)
+        my_console.download()
+        my_console.quit()
+        serial.s_close()
+        if apid:
+            self.com_api.ctl_connect()
+        try:
+            time.sleep(5.5)
+            print(self.board_id)
+        except (Reset_Exception, Max_Retry_Exception) as e:
+            pass
+
+
+
     def _dict_repr(self):
         dict = OrderedDict()
         dict['api_version']         = self.api_version
@@ -920,7 +1022,7 @@ class ChipSHOUTER(DisableNewAttr):
         dict['voltage']             = self.voltage._dict_repr()
         dict['pulse']               = self.pulse._dict_repr()
         dict['state']               = self.state
-        dict['trigger_safe']        = self.trigger_safe
+        #dict['trigger_safe']        = self.trigger_safe
         dict['faults_current']      = self.faults_current
         dict['faults_latched']      = self.faults_latched
         dict['temperature_mosfet']  = self.temperature_mosfet
