@@ -136,7 +136,7 @@ class Connection(object):
 
             # End logging 
             if Connection.START_UP_STRING in data:
-                raise Reset_Exception('Shouter has reset') 
+                raise Reset_Exception('ChipSHOUTER unit has reset - wait 5 seconds then reinitialize.') 
 
             if start < 0 or end < 0 or end < start:
                 b = bytearray()
@@ -527,6 +527,7 @@ class BP_TOOL(Connection):
     PULSE        = 5
     TRIGGER_SAFE = 6
     IS_RESET     = 7
+    EEPROM_WRITE = 8
     TIMEOUT      = 0xfc
     ACK          = 0x15 
     NACK         = 0xff 
@@ -882,6 +883,7 @@ class Protocol(BP_TOOL):
         self.to_follow = 0
         self.validcommands = [
             BP_TOOL.ARM,         
+            BP_TOOL.EEPROM_WRITE,
             BP_TOOL.DISARM,      
             BP_TOOL.DEFAULT,     
             BP_TOOL.RESET,       
@@ -1120,6 +1122,8 @@ class Bin_API(Protocol):
         rval = self.send_command_to_shouter(BP_TOOL.ARM)
         if rval != BP_TOOL.ARM:
             raise Firmware_State_Exception("State:" + self.get_state() + str(self.get_faults_latched()))
+    def cmd_eeprom_write(self, timeout=0):
+        self.send_command_to_shouter(BP_TOOL.EEPROM_WRITE)
     
     def cmd_pulse(self, timeout = 0):
         self.send_command_to_shouter(BP_TOOL.PULSE)
@@ -1149,6 +1153,7 @@ class Bin_API(Protocol):
 
     def cmd_reset(self, timeout = 0):
         self.send_command_to_shouter(BP_TOOL.RESET)
+        time.sleep(5)
         return self.ready_for_commands()
 
     def get_board_id(self, timeout = 0):
